package rwi.eclipse.team.ccvs.ssl;

/*******************************************************************************
 * Copyright (c) 2004 Rolf Wilms (rolf_wilms@hotmail.com).
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 * 
 *******************************************************************************/


import java.io.FilterInputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class StreamTracer {
	private static final int NA = 0; 
	private static final int IN = 1; 
	private static final int OUT = 2;
	
	private int prefixState = NA; 
	
	private OutputStream tracelogOutputStream;
	private TracedInputStream tracedInputStream;
	private TracedOutputStream tracedOutputStream;
	private String inPrefix;
	private String outPrefix;
	
	public StreamTracer(OutputStream tracelogOutputStream, InputStream inputStream, String inPrefix, OutputStream outputStream, String outPrefix) {
		this.tracelogOutputStream = tracelogOutputStream;
		this.tracedInputStream = new TracedInputStream(inputStream);
		this.tracedOutputStream = new TracedOutputStream(outputStream);
		this.inPrefix = inPrefix;
		this.outPrefix = outPrefix;
	}
	
	private class TracedOutputStream extends FilterOutputStream {
		public TracedOutputStream(OutputStream out) {
			super(out);
		}

		/* (non-Javadoc)
 		* @see java.io.FilterOutputStream#write(int)
 		*/
		public void write(int b) throws IOException {
			super.write(b);
			trace(OUT, b);
		}

		/* (non-Javadoc)
 		* @see java.io.FilterOutputStream#write(byte[], int, int)
 		*/
		public void write(byte[] b, int off, int len) throws IOException {
			out.write(b, off, len);
			trace(OUT, b, off, len);
		}
		
	}
	
	private class TracedInputStream extends FilterInputStream {
		public TracedInputStream(InputStream in) {
			super(in);
		}
		
		/* (non-Javadoc)
 		* @see java.io.FilterInputStream#read()
 		*/
		public int read() throws IOException {
			int b = super.read();
			trace(IN, b);
			return b;
		}

		/* (non-Javadoc)
 		* @see java.io.FilterInputStream#read(byte[], int, int)
 		*/
		public int read(byte[] b, int off, int len) throws IOException {
			int cnt = super.read(b, off, len);
			trace(IN, b, off, cnt); 
			return cnt;
		}
		
	}

	private synchronized void trace(int inout, byte[] b, int off, int len) throws IOException {
		checkPrefix(inout);
		tracelogOutputStream.write(b, off, len);
	}	

	private synchronized void trace(int inout, int b) throws IOException {
		if(b >= 0) {
			checkPrefix(inout);
			tracelogOutputStream.write(b);			
		}
	}	
	
	private void checkPrefix(int inout) throws IOException {
		if(prefixState == inout) {
			return;
		} 
		
		prefixState = inout;
		
		if(prefixState == IN) {
			tracelogOutputStream.write(inPrefix.getBytes());
		}  else if(prefixState == OUT) {
			tracelogOutputStream.write(outPrefix.getBytes());
		}
	} 

	public InputStream getTracedInputStream() {
		return tracedInputStream;
	}

	public OutputStream getTracedOutputStream() {
		return tracedOutputStream;
	}

}
